import random
import string

from typing import Optional
from datetime import datetime
from peewee import SqliteDatabase, Model, DateTimeField, CharField, IntegerField


def generate_short_url():
    # 生成短链接
    while True:
        short_url = "".join(random.sample(string.ascii_letters + string.digits, 6))
        if not check_short_id(short_url):
            return short_url


db = SqliteDatabase("database.db")


class BaseModel(Model):
    class Meta:
        database = db


class UniPay(BaseModel):
    created_at = DateTimeField(default=datetime.now)
    short_id = CharField(unique=True, default=generate_short_url)
    alipay = CharField()
    wechatpay = CharField()
    scan_count = IntegerField(default=0)

    class Meta:
        db_table = "unipay"


db.create_tables([UniPay])


def check_short_id(short_id):
    return UniPay.get_or_none(UniPay.short_id == short_id)


def add_unipay(alipay, wechatpay) -> UniPay:
    return UniPay.create(alipay=alipay, wechatpay=wechatpay)


def get_unipay(short_id) -> Optional[UniPay]:
    unipay: UniPay | None = UniPay.get_or_none(UniPay.short_id == short_id)
    if unipay:
        unipay.scan_count += 1
        unipay.save()
    return unipay


import qrcode
import base64

from pathlib import Path
from starlette import status
from fastapi import FastAPI, HTTPException, Request
from fastapi.responses import RedirectResponse
from pydantic import BaseModel
from fastapi.templating import Jinja2Templates


from .db import get_unipay, add_unipay


app = FastAPI()
templates = Jinja2Templates(directory=Path(__file__).parent / "templates")


class Unipay(BaseModel):
    short_id: str
    alipay: str
    wechatpay: str
    scan_count: int


class UnipayCreate(BaseModel):
    alipay: str
    wechatpay: str


def generate_qr(data: str) -> str:
    """Generate a data URI for the given data using QR code."""
    img = qrcode.make(data)

    # Convert the image to PNG and then to a Data URI
    from io import BytesIO

    buffered = BytesIO()
    img.save(buffered)
    img_str = base64.b64encode(buffered.getvalue()).decode()
    return f"data:image/png;base64,{img_str}"


@app.get("/")
async def root():
    return {"message": "Hello World"}


from fastapi import Request


@app.get("/{shortid}")
async def get_unipay_by_id(request: Request, shortid: str):
    unipay = get_unipay(shortid)

    if not unipay:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Unipay not found")

    user_agent = request.headers.get("User-Agent", "").lower()

    # If Alipay client, redirect to Alipay URL
    if "alipayclient" in user_agent:
        return RedirectResponse(url=unipay.alipay)

    alipay_qr = generate_qr(unipay.alipay)
    wechatpay_qr = generate_qr(unipay.wechatpay)

    # If WeChat client, show only WeChat QR code
    if "micromessenger" in user_agent:
        return templates.TemplateResponse("wechatpay.html", {"request": request, "wechatpay_qr": wechatpay_qr})

    # For other clients, show both QR codes
    return templates.TemplateResponse(
        "unipay.html", {"request": request, "alipay_qr": alipay_qr, "wechatpay_qr": wechatpay_qr}
    )


@app.put("/create")
async def create_unipay(unipay_req: UnipayCreate):
    unipay = add_unipay(unipay_req.alipay, unipay_req.wechatpay)
    return Unipay(
        short_id=unipay.short_id, alipay=unipay.alipay, wechatpay=unipay.wechatpay, scan_count=unipay.scan_count
    )
